@@ -65,6 +65,17 @@ if lgr.getEffectiveLevel() <= 1:
 __docformat__ = 'restructuredtext'
 
 
+def process_www_authenticate(v):
+    if not v:
+        return []
+    supported_type = v.split(' ')[0].lower()
+    our_type = {
+        'basic': 'http_basic_auth',
+        'digest': 'http_digest_auth',
+    }.get(supported_type)
+    return [our_type] if our_type else []
+
+
 def check_response_status(response, err_prefix="", session=None):
     """Check if response's status_code signals problem with authentication etc
 
@@ -81,7 +92,10 @@ def check_response_status(response, err_prefix="", session=None):
         # TODO: actually may be that is where we could use tagid and actually determine the form submission url
         raise DownloadError(err_prefix + "not found")
     elif 400 <= response.status_code < 500:
-        raise AccessDeniedError(err_msg)
+        raise AccessDeniedError(
+            err_msg,
+            supported_types=process_www_authenticate(
+                response.headers.get('WWW-Authenticate')))
     elif response.status_code in {200}:
         pass
     elif response.status_code in {301, 302, 307}:
@@ -129,14 +143,19 @@ class HTTPBaseAuthenticator(Authenticator):
         post_url = self.url if self.url else url
         credentials = credential()
 
-        # TODO: With digestauth there is no POST strictly necessary.
         # The whole thing relies on server first spitting out 401
         # and client GETing again with 'Authentication:' header
         # So we need custom handling for those, while keeping track not
         # of cookies per se, but of 'Authentication:' header which is
         # to be used in subsequent GETs
         response = self._post_credential(credentials, post_url, session)
+        if response is None:
+            # authentication did not involve any interaction, nothing to
+            # check at this point
+            return
 
+        # Handle responses if there was initial authentication exchange,
+        # e.g. posting to a form and getting a cookie etc
         err_prefix = "Authentication to %s failed: " % post_url
         try:
             check_response_status(response, err_prefix, session=session)
@@ -177,6 +196,7 @@ class HTTPBaseAuthenticator(Authenticator):
             for c, v in cookies_dict.items():
                 if c not in session.cookies or session.cookies[c] != v:
                     session.cookies[c] = v  # .update(cookies_dict)
+
         return response
 
     def _post_credential(self, credentials, post_url, session):
@@ -241,7 +261,8 @@ class HTMLFormAuthenticator(HTTPBaseAuthenticator):
 
         response = session.post(post_url, data=post_fields)
         lgr.debug("Posted to %s fields %s, got response %s with headers %s",
-                  post_url, list(post_fields.keys()), response, list(response.headers.keys()))
+                  post_url, list(post_fields.keys()), response,
+                  list(response.headers.keys()))
         return response
 
 
@@ -258,10 +279,9 @@ class HTTPRequestsAuthenticator(HTTPBaseAuthenticator):
         super(HTTPRequestsAuthenticator, self).__init__(**kwargs)
 
     def _post_credential(self, credentials, post_url, session):
-        response = session.post(post_url, data={},
-                                auth=self.REQUESTS_AUTHENTICATOR(
-                                    *[credentials[f] for f in self.REQUESTS_FIELDS]))
-        return response
+        authenticator = self.REQUESTS_AUTHENTICATOR(
+            *[credentials[f] for f in self.REQUESTS_FIELDS])
+        session.auth = authenticator
 
 
 @auto_repr
@@ -273,7 +293,6 @@ class HTTPBasicAuthAuthenticator(HTTPRequestsAuthenticator):
     ...
     credential = hcp-db
     authentication_type = http_auth
-    http_auth_url = .... TODO
 
     Parameters
     ----------
@@ -284,6 +303,25 @@ class HTTPBasicAuthAuthenticator(HTTPRequestsAuthenticator):
     REQUESTS_AUTHENTICATOR = requests.auth.HTTPBasicAuth
 
 
+@auto_repr
+class HTTPAuthAuthenticator(HTTPRequestsAuthenticator):
+    """Authenticate via Basic authentication to some other post url
+
+    TODO:  actually this is some remnants which might later were RFed
+    into the form authenticator since otherwise they make little sense
+    """
+
+    REQUESTS_AUTHENTICATOR = requests.auth.HTTPBasicAuth
+
+    def _post_credential(self, credentials, post_url, session):
+        authenticator = self.REQUESTS_AUTHENTICATOR(
+            *[credentials[f] for f in self.REQUESTS_FIELDS])
+        session.auth = authenticator
+        response = session.post(post_url, data={},
+                                auth=authenticator)
+        return response
+
+
 @auto_repr
 class HTTPDigestAuthAuthenticator(HTTPRequestsAuthenticator):
     """Authenticate via HTTP digest authentication
@@ -291,14 +329,14 @@ class HTTPDigestAuthAuthenticator(HTTPRequestsAuthenticator):
 
     REQUESTS_AUTHENTICATOR = requests.auth.HTTPDigestAuth
 
-    def _post_credential(self, *args, **kwargs):
-        raise NotImplementedError("not yet functioning, see https://github.com/kennethreitz/requests/issues/2934")
-
 
 @auto_repr
 class HTTPBearerTokenAuthenticator(HTTPRequestsAuthenticator):
     """Authenticate via HTTP Authorization header
     """
+
+    DEFAULT_CREDENTIAL_TYPE = 'token'
+
     def __init__(self, **kwargs):
         # so we have __init__ solely for a custom docstring
         super(HTTPBearerTokenAuthenticator, self).__init__(**kwargs)
@@ -306,12 +344,6 @@ class HTTPBearerTokenAuthenticator(HTTPRequestsAuthenticator):
     def _post_credential(self, credentials, post_url, session):
         # we do not need to post anything, just inject token into the session
         session.headers['Authorization'] = "Bearer %s" % credentials['token']
-        # TODO: actually access some location to verify that token is correct
-        class DummyResponse(object):
-            text = ''
-            cookies = None
-            status_code = 200
-        return DummyResponse()
 
 
 @auto_repr
@@ -460,17 +492,23 @@ class HTTPDownloader(BaseDownloader):
                     headers=headers)
             #except (MaxRetryError, NewConnectionError) as exc:
             except Exception as exc:
-                # happen to run into those with urls pointing to Amazon, so let's rest and try again
+                # happen to run into those with urls pointing to Amazon,
+                # so let's rest and try again
                 if retry >= nretries:
                     #import epdb; epdb.serve()
-                    raise AccessFailedError("Failed to establish a new session %d times. Last exception was: %s"
-                                            % (nretries, exc_str(exc)))
-                lgr.warning("Caught exception %s. Will retry %d out of %d times", exc_str(exc), retry+1, nretries)
+                    raise AccessFailedError(
+                        "Failed to establish a new session %d times. "
+                        "Last exception was: %s"
+                        % (nretries, exc_str(exc)))
+                lgr.warning(
+                    "Caught exception %s. Will retry %d out of %d times",
+                    exc_str(exc), retry+1, nretries)
                 sleep(2**retry)
 
         check_response_status(response, session=self._session)
         headers = response.headers
-        lgr.debug("Establishing session for url %s, response headers: %s", url, headers)
+        lgr.debug("Establishing session for url %s, response headers: %s",
+                  url, headers)
         target_size = int(headers.get('Content-Length', '0').strip()) or None
         if use_redirected_url and response.url and response.url != url:
             lgr.debug("URL %s was redirected to %s and thus the later will be used"
