@@ -13,7 +13,7 @@
 __docformat__ = 'restructuredtext'
 
 from appdirs import AppDirs
-from os.path import join as opj
+from os.path import join as opj, expanduser
 from datalad.support.constraints import EnsureBool
 from datalad.support.constraints import EnsureInt
 from datalad.support.constraints import EnsureNone
@@ -44,6 +44,14 @@ definitions = {
         'destination': 'global',
         'default': dirs.user_cache_dir,
     },
+    'datalad.locations.default-dataset': {
+        'ui': ('question', {
+               'title': 'Default dataset path',
+               'text': 'Where should datalad should look for (or install) a '
+                       'default dataset?'}),
+        'destination': 'global',
+        'default': opj(expanduser('~'), 'datalad'),
+    },
     'datalad.locations.system-plugins': {
         'ui': ('question', {
                'title': 'System plugin directory',
