@@ -33,7 +33,6 @@ from datalad.interface.run import _format_cmd_shorty
 from datalad.consts import PRE_INIT_COMMIT_SHA
 
 from datalad.support.constraints import EnsureNone, EnsureStr
-from datalad.support.gitrepo import GitCommandError
 from datalad.support.param import Parameter
 from datalad.support.json_py import load_stream
 
@@ -212,7 +211,7 @@ class Rerun(Interface):
                 message="branch '{}' already exists".format(branch))
             return
 
-        if not commit_exists(ds, revision + "^"):
+        if not ds.repo.commit_exists(revision + "^"):
             # Only a single commit is reachable from `revision`.  In
             # this case, --since has no effect on the range construction.
             revrange = revision
@@ -244,7 +243,7 @@ class Rerun(Interface):
 def _revs_as_results(dset, revs):
     for rev in revs:
         res = get_status_dict("run", ds=dset, commit=rev)
-        full_msg = dset.repo.repo.git.show(rev, "--format=%B", "--no-patch")
+        full_msg = dset.repo.format_commit("%B", rev)
         try:
             msg, info = get_run_info(dset, full_msg)
         except ValueError as exc:
@@ -294,7 +293,7 @@ def _rerun_as_results(dset, revrange, since, branch, onto, message):
         # that, regardless of if and how --since is specified, the effective
         # value for --since is the parent of the first revision.
         onto = results[0]["commit"] + "^"
-        if not commit_exists(dset, onto):
+        if not dset.repo.commit_exists(onto):
             # This is unlikely to happen in the wild because it means that the
             # first commit is a datalad run commit. Just abort rather than
             # trying to checkout on orphan branch or something like that.
@@ -314,12 +313,7 @@ def _rerun_as_results(dset, revrange, since, branch, onto, message):
             status="ok")
 
     def rev_is_ancestor(rev):
-        try:
-            dset.repo.repo.git.merge_base("--is-ancestor", rev, start_point)
-        except GitCommandError:
-            # Revision is NOT an ancestor of the starting point.
-            return False
-        return True
+        return dset.repo.is_ancestor(rev, start_point)
 
     # We want to skip revs before the starting point and pick those after.
     to_pick = set(dropwhile(rev_is_ancestor, [r["commit"] for r in results]))
@@ -327,7 +321,7 @@ def _rerun_as_results(dset, revrange, since, branch, onto, message):
     def skip_or_pick(hexsha, result, msg):
         pick = hexsha in to_pick
         result["rerun_action"] = "pick" if pick else "skip"
-        shortrev = dset.repo.repo.git.rev_parse("--short", hexsha)
+        shortrev = dset.repo.get_hexsha(hexsha, short=True)
         result["message"] = (
             "%s %s; %s",
             shortrev, msg, "cherry picking" if pick else "skipping")
@@ -364,9 +358,7 @@ def _rerun(dset, results):
             dset.repo.checkout(res["commit"],
                                options=checkout_options)
         elif rerun_action == "pick":
-            dset.repo._git_custom_command(
-                None, ["git", "cherry-pick", res["commit"]],
-                check_fake_dates=True)
+            dset.repo.cherry_pick(res["commit"])
             yield res
         elif rerun_action == "run":
             hexsha = res["commit"]
@@ -407,8 +399,7 @@ def _report(dset, results):
                 res["diff"] = list(res["diff"])
                 # Add extra information that is useful in the report but not
                 # needed for the rerun.
-                out = dset.repo.repo.git.show(
-                    "--no-patch", "--format=%an%x00%aI", res["commit"])
+                out = dset.repo.format_commit("%an%x00%aI", res["commit"])
                 res["author"], res["date"] = out.split("\0")
         yield res
 
@@ -428,7 +419,7 @@ def _get_script_handler(script, since, revision):
         ofh.write(header.format(
             script=script,
             since="" if since is None else " --since=" + since,
-            revision=dset.repo.repo.git.rev_parse(revision),
+            revision=dset.repo.get_hexsha(revision),
             ds='dataset {} at '.format(dset.id) if dset.id else '',
             path=dset.path))
 
@@ -537,7 +528,7 @@ def diff_revision(dataset, revision="HEAD"):
     -------
     Generator that yields AnnotatePaths instances
     """
-    if commit_exists(dataset, revision + "^"):
+    if dataset.repo.commit_exists(revision + "^"):
         revrange = "{rev}^..{rev}".format(rev=revision)
     else:
         # No other commits are reachable from this revision.  Diff
@@ -557,11 +548,3 @@ def new_or_modified(diff_results):
         if r.get('type') == 'file' and r.get('state') in ['added', 'modified']:
             r.pop('status', None)
             yield r
-
-
-def commit_exists(dataset, commit):
-    try:
-        dataset.repo.repo.git.rev_parse("--verify", commit + "^{commit}")
-    except:
-        return False
-    return True
