@@ -22,7 +22,6 @@ from ...tests.utils import ok_, ok_exists, eq_, assert_cwd_unchanged, \
     with_tempfile
 from ...tests.utils import with_tree
 from ...tests.utils import serve_path_via_http
-from ...tests.utils import known_failure_direct_mode
 
 
 def test_download_url_exceptions():
@@ -80,7 +79,6 @@ def test_download_url_return(toppath, topurl, outdir):
 ])
 @serve_path_via_http
 @with_tempfile(mkdir=True)
-@known_failure_direct_mode  #FIXME
 def test_download_url_dataset(toppath, topurl, path):
     # Non-dataset directory.
     file1_fullpath = opj(path, "file1.txt")
@@ -123,7 +121,6 @@ def test_download_url_dataset(toppath, topurl, path):
 @with_tree(tree={"archive.tar.gz": {'file1.txt': 'abc'}})
 @serve_path_via_http
 @with_tempfile(mkdir=True)
-@known_failure_direct_mode  #FIXME
 def test_download_url_archive(toppath, topurl, path):
     ds = Dataset(path).create()
     ds.download_url([opj(topurl, "archive.tar.gz")], archive=True)
