@@ -29,7 +29,6 @@ from datalad.tests.utils import chpwd
 from datalad.tests.utils import assert_cwd_unchanged
 from datalad.tests.utils import with_testrepos
 from datalad.tests.utils import on_windows, skip_if
-from datalad.tests.utils import known_failure_v6
 from datalad.tests.utils import assert_status, assert_result_count, assert_in_results
 
 
@@ -66,7 +65,6 @@ def test_unlock_raises(path, path2, path3):
     chpwd(_cwd)
 
 
-@known_failure_v6  # FIXME: See TODOs in the comments below
 # Note: As root there is no actual lock/unlock.
 #       Therefore don't know what to test for yet.
 @skip_if(cond=not on_windows and os.geteuid() == 0)  # uid not available on windows
