@@ -179,7 +179,7 @@ def discover_dataset_trace_to_targets(basepath, targetpaths, current_trace,
       Path to a start or top-level dataset. Really has to be a path to a
       dataset!
     targetpaths : list(path)
-      Any non-zero number of path that are termination points for the
+      Any non-zero number of paths that are termination points for the
       search algorithm. Can be paths to datasets, directories, or files
       (and any combination thereof).
     current_trace : list
@@ -548,24 +548,26 @@ def _process_results(
             pass
         elif result_renderer == 'default':
             # TODO have a helper that can expand a result message
-            ui.message('{action}({status}): {path}{type}{msg}'.format(
-                action=ac.color_word(res['action'], ac.BOLD),
-                status=ac.color_status(res['status']),
-                path=relpath(res['path'],
-                             res['refds']) if res.get('refds', None) else res['path'],
-                type=' ({})'.format(
-                    ac.color_word(res['type'], ac.MAGENTA)
+            if res.get('status', None) != 'notneeded':
+                ui.message('{action}({status}): {path}{type}{msg}'.format(
+                    action=ac.color_word(res['action'], ac.BOLD),
+                    status=ac.color_status(res['status']),
+                    path=relpath(res['path'],
+                                 res['refds']) if res.get('refds', None) else res['path'],
+                    type=' ({})'.format(
+                        ac.color_word(res['type'], ac.MAGENTA)
                     ) if 'type' in res else '',
-                msg=' [{}]'.format(
-                    res['message'][0] % res['message'][1:]
-                    if isinstance(res['message'], tuple) else res['message'])
-                if 'message' in res else ''))
+                    msg=' [{}]'.format(
+                        res['message'][0] % res['message'][1:]
+                        if isinstance(res['message'], tuple) else res['message'])
+                    if 'message' in res else ''))
         elif result_renderer in ('json', 'json_pp'):
             ui.message(json.dumps(
                 {k: v for k, v in res.items()
                  if k not in ('message', 'logger')},
                 sort_keys=True,
-                indent=2 if result_renderer.endswith('_pp') else None))
+                indent=2 if result_renderer.endswith('_pp') else None,
+                default=lambda x: str(x)))
         elif result_renderer == 'tailored':
             if hasattr(cmd_class, 'custom_result_renderer'):
                 cmd_class.custom_result_renderer(res, **kwargs)
