@@ -40,6 +40,7 @@ from six import string_types
 from six import add_metaclass
 from functools import wraps
 import git as gitpy
+from git import RemoteProgress
 from gitdb.exc import BadName
 from git.exc import GitCommandError
 from git.exc import NoSuchPathError
@@ -57,6 +58,7 @@ from datalad.utils import on_windows
 from datalad.utils import getpwd
 from datalad.utils import updated
 from datalad.utils import posix_relpath
+from datalad.utils import assure_dir
 from ..utils import assure_unicode
 
 # imports from same module:
@@ -442,6 +444,95 @@ def guard_BadName(func):
     return wrapped
 
 
+class GitPythonProgressBar(RemoteProgress):
+    """A handler for Git commands interfaced by GitPython which report progress
+    """
+
+    # GitPython operates with op_codes which are a mask for actions.
+    _known_ops = {
+        RemoteProgress.COUNTING: "counting objects",
+        RemoteProgress.COMPRESSING: "compressing objects",
+        RemoteProgress.WRITING: "writing objects",
+        RemoteProgress.RECEIVING: "receiving objects",
+        RemoteProgress.RESOLVING: "resolving stuff",
+        RemoteProgress.FINDING_SOURCES: "finding sources",
+        RemoteProgress.CHECKING_OUT: "checking things out"
+    }
+
+    # To overcome the bug when GitPython (<=2.1.11), with tentative fix
+    # in https://github.com/gitpython-developers/GitPython/pull/798
+    # we will collect error_lines from the last progress bar used by GitPython
+    # To do that reliably this class should be used as a ContextManager,
+    # or .close() should be called explicitly before analysis of this
+    # attribute is done.
+    # TODO: remove the workaround whenever new GitPython version provides
+    # it natively and we boost versioned dependency on it
+    _last_error_lines = None
+
+    def __init__(self, action):
+        super(GitPythonProgressBar, self).__init__()
+        self._action = action
+        from datalad.ui import ui
+        self._ui = ui
+        self._pbar = None
+        self._op_code = None
+        GitPythonProgressBar._last_error_lines = None
+
+    def __del__(self):
+        self.close()
+
+    def close(self):
+        GitPythonProgressBar._last_error_lines = self.error_lines
+        self._close_pbar()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
+    def _close_pbar(self):
+        if self._pbar:
+            self._pbar.finish()
+        self._pbar = None
+
+    def _get_human_msg(self, op_code):
+        """Return human readable action message
+        """
+        op_id = op_code & self.OP_MASK
+        op = self._known_ops.get(op_id, "doing other evil")
+        return "%s (%s)" % (self._action, op)
+
+    def update(self, op_code, cur_count, max_count=None, message=''):
+        # ATM we ignore message which typically includes bandwidth info etc
+        try:
+            if not max_count:
+                # spotted used by GitPython tests, so may be at times it is not
+                # known and assumed to be a 100%...? TODO
+                max_count = 100.0
+            if self._op_code is None or self._op_code != op_code:
+                # new type of operation
+                self._close_pbar()
+
+                self._pbar = self._ui.get_progressbar(
+                    self._get_human_msg(op_code),
+                    total=max_count
+                )
+                self._op_code = op_code
+            if not self._pbar:
+                lgr.error("Ended up without progress bar... how?")
+                return
+            self._pbar.update(cur_count, increment=False)
+        except Exception as exc:
+            lgr.debug("GitPythonProgressBar errored with %s", exc_str(exc))
+            return
+        #import time; time.sleep(0.001)  # to see that things are actually "moving"
+        # without it we would get only a blink on initial 0 value, istead of
+        # a blink at some higher value.  Anyways git provides those
+        # without flooding so should be safe to force here.
+        self._pbar.refresh()
+
+
 @add_metaclass(Flyweight)
 class GitRepo(RepoInterface):
     """Representation of a git repository
@@ -451,11 +542,6 @@ class GitRepo(RepoInterface):
     # We use our sshrun helper
     GIT_SSH_ENV = {'GIT_SSH_COMMAND': GIT_SSH_COMMAND}
 
-    # Just a non-functional example:
-    # must be implemented, since abstract in RepoInterface:
-    def sth_like_file_has_content(self):
-        return "Yes, if it's in the index"
-
     # We must check git config to have name and email set, but
     # should do it once
     _config_checked = False
@@ -735,8 +821,13 @@ class GitRepo(RepoInterface):
         for trial in range(ntries):
             try:
                 lgr.debug("Git clone from {0} to {1}".format(url, path))
-                repo = gitpy.Repo.clone_from(url, path, env=env,
-                                             odbt=default_git_odbt)
+                with GitPythonProgressBar("Cloning") as git_progress:
+                    repo = gitpy.Repo.clone_from(
+                        url, path,
+                        env=env,
+                        odbt=default_git_odbt,
+                        progress=git_progress
+                    )
                 # Note/TODO: signature for clone from:
                 # (url, to_path, progress=None, env=None, **kwargs)
 
@@ -777,11 +868,10 @@ class GitRepo(RepoInterface):
         # Make sure to flush pending changes, especially close batch processes
         # (internal `git cat-file --batch` by GitPython)
         try:
-            if hasattr(self, 'repo') and exists(self.path) \
-                    and self.repo is not None:
+            if getattr(self, '_repo', None) is not None and exists(self.path):
                 # gc might be late, so the (temporary)
                 # repo doesn't exist on FS anymore
-                self.repo.git.clear_cache()
+                self._repo.git.clear_cache()
                 # We used to write out the index to flush GitPython's
                 # state... but such unconditional write is really a workaround
                 # and does not play nice with read-only operations - permission
@@ -807,6 +897,44 @@ class GitRepo(RepoInterface):
         """Returns if a given path points to a git repository"""
         return exists(opj(path, '.git'))
 
+    @staticmethod
+    def get_git_dir(repo):
+        """figure out a repo's gitdir
+
+        '.git' might be a  directory, a symlink or a file
+
+        Parameter
+        ---------
+        repo: path or Repo instance
+          currently expected to be the repos base dir
+
+        Returns
+        -------
+        str
+          relative path to the repo's git dir; So, default would be ".git"
+        """
+        if hasattr(repo, 'path'):
+            # repo instance like given
+            repo = repo.path
+        dot_git = op.join(repo, ".git")
+        if not op.exists(dot_git):
+            raise RuntimeError("Missing .git in %s." % repo)
+        elif op.islink(dot_git):
+            # readlink cannot be imported on windows, but there should also
+            # be no symlinks
+            from os import readlink
+            git_dir = readlink(dot_git)
+        elif op.isdir(dot_git):
+            git_dir = ".git"
+        elif op.isfile(dot_git):
+            with open(dot_git) as f:
+                git_dir = f.readline()
+                if git_dir.startswith("gitdir:"):
+                    git_dir = git_dir[7:]
+                git_dir = git_dir.strip()
+
+        return git_dir
+
     @property
     def config(self):
         """Get an instance of the parser for the persistent repository
@@ -1213,17 +1341,23 @@ class GitRepo(RepoInterface):
         return [x[0] for x in self.cmd_call_wrapper(
             self.repo.index.entries.keys)]
 
-    def get_hexsha(self, object=None):
-        """Return a hexsha for a given object. If None - of current HEAD
+    def format_commit(self, fmt, commitish=None):
+        """Return `git show` output for `commitish`.
 
         Parameters
         ----------
-        object: str, optional
-          Any type of Git object identifier. See `git show`.
+        fmt : str
+            A format string accepted by `git show`.
+        commitish: str, optional
+          Any commit identifier (defaults to "HEAD").
+
+        Returns
+        -------
+        str or, if there are not commits yet, None.
         """
-        cmd = ['git', 'show', '--no-patch', "--format=%H"]
-        if object:
-            cmd.append(object)
+        cmd = ['git', 'show', '-z', '--no-patch', '--format=' + fmt]
+        if commitish is not None:
+            cmd.append(commitish + "^{commit}")
         # make sure Git takes our argument as a revision
         cmd.append('--')
         try:
@@ -1231,14 +1365,37 @@ class GitRepo(RepoInterface):
                 '', cmd, expect_stderr=True, expect_fail=True)
         except CommandError as e:
             if 'bad revision' in e.stderr:
-                raise ValueError("Unknown object identifier: %s" % object)
+                raise ValueError("Unknown commit identifier: %s" % commitish)
             elif 'does not have any commits yet' in e.stderr:
                 return None
             else:
                 raise e
-        stdout = stdout.splitlines()
-        assert(len(stdout) == 1)
-        return stdout[0]
+        # This trailing null is coming from the -z above, which avoids the
+        # newline that Git would append to the output. We could drop -z and
+        # strip the newline directly, but then we'd have to worry about
+        # compatibility across platforms.
+        return stdout.rsplit("\0", 1)[0]
+
+    def get_hexsha(self, commitish=None, short=False):
+        """Return a hexsha for a given commitish.
+
+        Parameters
+        ----------
+        commitish : str, optional
+          Any identifier that refers to a commit (defaults to "HEAD").
+        short : bool, optional
+          Return the abbreviated form of the hexsha.
+
+        Returns
+        -------
+        str or, if there are not commits yet, None.
+        """
+        stdout = self.format_commit("%{}".format('h' if short else 'H'),
+                                    commitish)
+        if stdout is not None:
+            stdout = stdout.splitlines()
+            assert(len(stdout) == 1)
+            return stdout[0]
 
     @normalize_paths(match_return_type=False)
     def get_last_commit_hash(self, files):
@@ -1256,13 +1413,36 @@ class GitRepo(RepoInterface):
                 return None
             raise
 
-    def get_merge_base(self, treeishes):
+    def commit_exists(self, commitish):
+        """Does `commitish` exist in the repo?
+
+        Parameters
+        ----------
+        commitish : str
+            A commit or an object that can be dereferenced to one.
+
+        Returns
+        -------
+        bool
+        """
+        try:
+            # Note: The peeling operator "^{commit}" is required so that
+            # rev-parse doesn't succeed if passed a full hexsha that is valid
+            # but doesn't exist.
+            self._git_custom_command(
+                "", ["git", "rev-parse", "--verify", commitish + "^{commit}"],
+                expect_fail=True)
+        except CommandError:
+            return False
+        return True
+
+    def get_merge_base(self, commitishes):
         """Get a merge base hexsha
 
         Parameters
         ----------
-        treeishes: str or list of str
-          List of treeishes (branches, hexshas, etc) to determine the merge
+        commitishes: str or list of str
+          List of commitishes (branches, hexshas, etc) to determine the merge
           base of. If a single value provided, returns merge_base with the
           current branch.
 
@@ -1272,15 +1452,15 @@ class GitRepo(RepoInterface):
           If no merge-base for given commits, or specified treeish doesn't
           exist, None returned
         """
-        if isinstance(treeishes, string_types):
-            treeishes = [treeishes]
-        if not treeishes:
+        if isinstance(commitishes, string_types):
+            commitishes = [commitishes]
+        if not commitishes:
             raise ValueError("Provide at least a single value")
-        elif len(treeishes) == 1:
-            treeishes = treeishes + [self.get_active_branch()]
+        elif len(commitishes) == 1:
+            commitishes = commitishes + [self.get_active_branch()]
 
         try:
-            bases = self.repo.merge_base(*treeishes)
+            bases = self.repo.merge_base(*commitishes)
         except GitCommandError as exc:
             if "fatal: Not a valid object name" in str(exc):
                 return None
@@ -1291,6 +1471,26 @@ class GitRepo(RepoInterface):
         assert(len(bases) == 1)  # we do not do 'all' yet
         return bases[0].hexsha
 
+    def is_ancestor(self, reva, revb):
+        """Is `reva` an ancestor of `revb`?
+
+        Parameters
+        ----------
+        reva, revb : str
+            Revisions.
+
+        Returns
+        -------
+        bool
+        """
+        try:
+            self._git_custom_command(
+                "", ["git", "merge-base", "--is-ancestor", reva, revb],
+                expect_fail=True)
+        except CommandError:
+            return False
+        return True
+
     def get_commit_date(self, branch=None, date='authored'):
         """Get the date stamp of the last commit (in a branch or head otherwise)
 
@@ -1480,101 +1680,6 @@ class GitRepo(RepoInterface):
             if any(rb.startswith(remote + '/') for rb in remote_branches)
         ]
 
-    def _gitpy_custom_call(self, cmd, cmd_args=None, cmd_options=None,
-                           git_options=None, env=None,
-
-                           # 'old' options for Runner; not sure yet, which of
-                           # them are actually still needed:
-                           log_stdout=True, log_stderr=True, log_online=False,
-                           expect_stderr=True, cwd=None,
-                           shell=None, expect_fail=False):
-
-        """Helper to call GitPython's wrapper for git calls.
-
-        The used instance of `gitpy.Git` is bound to the repository,
-        which determines its working directory.
-        This is used for adhoc implementation of a git command and to
-        demonstrate how to use it in more specific implementations.
-
-        Note
-        ----
-        Aims to replace the use of datalad's `Runner` class for direct git
-        calls. (Currently the `_git_custom_command()` method).
-        Therefore mimicking its behaviour during RF'ing.
-
-        Parameters
-        ----------
-        cmd: str
-          the native git command to call
-        cmd_args: list of str
-          arguments to the git command
-        cmd_options: dict
-          options for the command as key, value pair
-          (this transformation, needs some central place to document)
-        git_options: dict
-          options for the git executable as key, value pair
-          (see above)
-        env: dict
-          environment variables to temporarily set for this call
-
-        TODO
-        ----
-        Example
-
-        Returns
-        -------
-        (stdout, stderr)
-        """
-
-        # TODO: Reconsider when to log/stream what (stdout, stderr) and/or
-        # fully implement the behaviour of `Runner`
-
-        if log_online:
-            raise NotImplementedError("option 'log_online' not implemented yet")
-        with_exceptions = not expect_fail
-        if cwd:
-            # the gitpy.cmd.Git instance, bound to this repository doesn't allow
-            # to explicitly set the working dir, except for using os.getcwd
-            raise NotImplementedError("working dir is a read-only property")
-
-        _tmp_shell = gitpy.cmd.Git.USE_SHELL
-        gitpy.cmd.Git.USE_SHELL = shell
-
-        if env is None:
-            env = {}
-        if git_options is None:
-            git_options = {}
-        if cmd_options is None:
-            cmd_options = {}
-        cmd_options.update({'with_exceptions': with_exceptions,
-                            'with_extended_output': True})
-
-        # TODO: _GIT_COMMON_OPTIONS!
-
-        with self.repo.git.custom_environment(**env):
-            try:
-                status, std_out, std_err = \
-                    self.repo.git(**git_options).__getattr__(cmd)(
-                        cmd_args, **cmd_options)
-            except GitCommandError as e:
-                # For now just reraise. May be raise CommandError instead
-                raise
-            finally:
-                gitpy.cmd.Git.USE_SHELL = _tmp_shell
-
-        if not expect_stderr and std_err:
-            lgr.error("Unexpected output on stderr: %s" % std_err)
-            raise CommandError
-        if log_stdout:
-            for line in std_out.splitlines():
-                lgr.debug("stdout| " + line)
-        if log_stderr:
-            for line in std_err.splitlines():
-                lgr.log(level=logging.DEBUG if expect_stderr else logging.ERROR,
-                        msg="stderr| " + line)
-
-        return std_out, std_err
-
     @normalize_paths(match_return_type=False)
     def _git_custom_command(self, files, cmd_str,
                             log_stdout=True, log_stderr=True, log_online=False,
@@ -1693,8 +1798,7 @@ class GitRepo(RepoInterface):
     # TODO: centralize all the c&p code in fetch, pull, push
     # TODO: document **kwargs passed to gitpython
     @guard_BadName
-    def fetch(self, remote=None, refspec=None, progress=None, all_=False,
-              **kwargs):
+    def fetch(self, remote=None, refspec=None, all_=False, **kwargs):
         """Fetches changes from a remote (or all_ remotes).
 
         Parameters
@@ -1704,9 +1808,6 @@ class GitRepo(RepoInterface):
           `all_` is not set, the tracking branch is fetched.
         refspec: str
           (optional) refspec to fetch.
-        progress:
-          passed to gitpython. TODO: Figure it out, make consistent use of it
-          and document it.
         all_: bool
           fetch all_ remotes (and all_ of their branches).
           Fails if `remote` was given.
@@ -1759,23 +1860,49 @@ class GitRepo(RepoInterface):
                 lgr.debug("Remote %s has no URL", rm)
                 return []
 
-            if is_ssh(fetch_url):
-                ssh_manager.get_connection(fetch_url).open()
-                # TODO: with git <= 2.3 keep old mechanism:
-                #       with rm.repo.git.custom_environment(GIT_SSH="wrapper_script"):
-                with rm.repo.git.custom_environment(**GitRepo.GIT_SSH_ENV):
-                    fi_list += rm.fetch(refspec=refspec, progress=progress, **kwargs)
-                    # TODO: progress +kwargs
-            else:
-                fi_list += rm.fetch(refspec=refspec, progress=progress, **kwargs)
-                # TODO: progress +kwargs
+            fi_list += self._call_gitpy_with_progress(
+                "Fetching %s" % rm.name,
+                rm.fetch,
+                rm.repo,
+                refspec,
+                fetch_url,
+                **kwargs
+            )
 
         # TODO: fetch returns a list of FetchInfo instances. Make use of it.
         return fi_list
 
-    def pull(self, remote=None, refspec=None, progress=None, **kwargs):
+    def _call_gitpy_with_progress(self, msg, callable, git_repo,
+                                  refspec, url, **kwargs):
+        """A helper to reduce code duplication
+
+        Wraps call to a GitPython method with all needed decoration for
+        workarounds of having aged git, or not providing full stderr
+        when monitoring progress of the operation
+        """
+        with GitPythonProgressBar(msg) as git_progress:
+            git_kwargs = dict(
+                refspec=refspec,
+                progress=git_progress,
+                **kwargs
+            )
+            if is_ssh(url):
+                ssh_manager.get_connection(url).open()
+                # TODO: with git <= 2.3 keep old mechanism:
+                #       with rm.repo.git.custom_environment(
+                # GIT_SSH="wrapper_script"):
+                with git_repo.git.custom_environment(**GitRepo.GIT_SSH_ENV):
+                    ret = callable(**git_kwargs)
+                    # TODO: +kwargs
+            else:
+                ret = callable(**git_kwargs)
+                # TODO: +kwargs
+        return ret
+
+    def pull(self, remote=None, refspec=None, **kwargs):
         """See fetch
         """
+
         if remote is None:
             if refspec is not None:
                 # conflicts with using tracking branch or fetch all remotes
@@ -1801,18 +1928,17 @@ class GitRepo(RepoInterface):
             remote.config_reader.get(
                 'fetchurl' if remote.config_reader.has_option('fetchurl')
                 else 'url')
-        if is_ssh(fetch_url):
-            ssh_manager.get_connection(fetch_url).open()
-            # TODO: with git <= 2.3 keep old mechanism:
-            #       with remote.repo.git.custom_environment(GIT_SSH="wrapper_script"):
-            with remote.repo.git.custom_environment(**GitRepo.GIT_SSH_ENV):
-                return remote.pull(refspec=refspec, progress=progress, **kwargs)
-                # TODO: progress +kwargs
-        else:
-            return remote.pull(refspec=refspec, progress=progress, **kwargs)
-            # TODO: progress +kwargs
 
-    def push(self, remote=None, refspec=None, progress=None, all_remotes=False,
+        return self._call_gitpy_with_progress(
+                "Pulling",
+                remote.pull,
+                remote.repo,
+                refspec,
+                fetch_url,
+                **kwargs
+            )
+
+    def push(self, remote=None, refspec=None, all_remotes=False,
              **kwargs):
         """Push to remote repository
 
@@ -1822,8 +1948,6 @@ class GitRepo(RepoInterface):
           name of the remote to push to
         refspec: str
           specify what to push
-        progress:
-          TODO
         all_remotes: bool
           if set to True push to all remotes. Conflicts with `remote` not being
           None.
@@ -1883,16 +2007,14 @@ class GitRepo(RepoInterface):
                 rm.config_reader.get('pushurl'
                                      if rm.config_reader.has_option('pushurl')
                                      else 'url')
-            if is_ssh(push_url):
-                ssh_manager.get_connection(push_url).open()
-                # TODO: with git <= 2.3 keep old mechanism:
-                #       with rm.repo.git.custom_environment(GIT_SSH="wrapper_script"):
-                with rm.repo.git.custom_environment(**GitRepo.GIT_SSH_ENV):
-                    pi_list += rm.push(refspec=refspec, progress=progress, **kwargs)
-                    # TODO: progress +kwargs
-            else:
-                pi_list += rm.push(refspec=refspec, progress=progress, **kwargs)
-                # TODO: progress +kwargs
+            pi_list += self._call_gitpy_with_progress(
+                "Pushing %s" % rm.name,
+                rm.push,
+                rm.repo,
+                refspec,
+                push_url,
+                **kwargs
+            )
         return pi_list
 
     def get_remote_url(self, name, push=False):
@@ -2016,6 +2138,17 @@ class GitRepo(RepoInterface):
             '', ['git', 'branch', '-D', branch]
         )
 
+    def cherry_pick(self, commit):
+        """Cherry pick `commit` to the current branch.
+
+        Parameters
+        ----------
+        commit : str
+            A single commit.
+        """
+        self._git_custom_command("", ["git", "cherry-pick", commit],
+                                 check_fake_dates=True)
+
     def ls_remote(self, remote, options=None):
         if options is None:
             options = []
@@ -2147,6 +2280,21 @@ class GitRepo(RepoInterface):
         if branch is not None:
             cmd += ['-b', branch]
         if url is None:
+            # repo must already exist locally
+            subm = GitRepo(op.join(self.path, path), create=False, init=False)
+            # check that it has a commit, and refuse
+            # to operate on it otherwise, or we would get a bastard
+            # submodule that cripples git operations
+            if not subm.get_hexsha():
+                raise InvalidGitRepositoryError(
+                    'cannot add subdataset {} with no commits'.format(subm))
+            # make an attempt to configure a submodule source URL based on the
+            # discovered remote configuration
+            remote, branch = subm.get_tracking_branch()
+            url = subm.get_remote_url(remote) if remote else None
+
+        if url is None:
+            # had no luck with a remote URL
             if not isabs(path):
                 # need to recode into a relative path "URL" in POSIX
                 # style, even on windows
@@ -2155,6 +2303,8 @@ class GitRepo(RepoInterface):
                 url = path
         cmd += [url, path]
         self._git_custom_command('', cmd)
+        # ensure supported setup
+        _fixup_submodule_dotgit_setup(self, path)
         # TODO: return value
 
     def deinit_submodule(self, path, **kwargs):
@@ -2473,9 +2623,46 @@ class GitRepo(RepoInterface):
                         attrline += ' -{}'.format(a)
                     else:
                         attrline += ' {}={}'.format(a, val)
-                f.write('{}\n'.format(attrline))
+                f.write('\n{}'.format(attrline))
 
 
 # TODO
 # remove submodule: nope, this is just deinit_submodule + remove
 # status?
+
+
+def _fixup_submodule_dotgit_setup(ds, relativepath):
+    """Implementation of our current of .git in a subdataset
+
+    Each subdataset/module has its own .git directory where a standalone
+    repository would have it. No gitdir files, no symlinks.
+    """
+    # move .git to superrepo's .git/modules, remove .git, create
+    # .git-file
+    path = opj(ds.path, relativepath)
+    subds_dotgit = opj(path, ".git")
+    src_dotgit = GitRepo.get_git_dir(path)
+
+    if src_dotgit == '.git':
+        # this is what we want
+        return
+
+    # first we want to remove any conflicting worktree setup
+    # done by git to find the checkout at the mountpoint of the
+    # submodule, if we keep that, any git command will fail
+    # after we move .git
+    GitRepo(path, init=False).config.unset(
+        'core.worktree', where='local')
+    # what we have here is some kind of reference, remove and
+    # replace by the target
+    os.remove(subds_dotgit)
+    # make absolute
+    src_dotgit = opj(path, src_dotgit)
+    # move .git
+    from os import rename, listdir, rmdir
+    assure_dir(subds_dotgit)
+    for dot_git_entry in listdir(src_dotgit):
+        rename(opj(src_dotgit, dot_git_entry),
+               opj(subds_dotgit, dot_git_entry))
+    assert not listdir(src_dotgit)
+    rmdir(src_dotgit)
